@@ -5,8 +5,7 @@ from cwltool.main import main
 
 from .util import needs_docker
 
-
-script = '''
+script = """
 #!/usr/bin/env cwl-runner
 cwlVersion: v1.0
 class: CommandLineTool
@@ -22,12 +21,13 @@ outputs:
       glob: test.txt
 stdout: test.txt
 baseCommand: [cat]
-'''
+"""
+
 
 @needs_docker
-def test_spaces_in_input_files():
+def test_spaces_in_input_files(tmpdir):
     try:
-        script_file = NamedTemporaryFile(mode='w', delete=False)
+        script_file = NamedTemporaryFile(mode="w", delete=False)
         script_file.write(script)
         script_file.flush()
         script_file.close()
@@ -35,7 +35,14 @@ def test_spaces_in_input_files():
         spaces = NamedTemporaryFile(prefix="test with spaces", delete=False)
         spaces.close()
 
-        params = ["--debug", script_file.name, '--input', spaces.name]
+        params = [
+            "--debug",
+            "--outdir",
+            str(tmpdir),
+            script_file.name,
+            "--input",
+            spaces.name,
+        ]
         assert main(params) == 1
         assert main(["--relax-path-checks"] + params) == 0
     finally:
